#!/usr/bin/perl -w
# topbull07.pl
# Auteur: Jean-Marie Pacquet
#
# Pour une date donne (option "-d aaaa-mm-jj") il calcule:
# 1. quel joueur a gagn le plus de places sur la priode
# 2. quel joueur a perdu le plus de places sur la priode
# il transfre via FTP (option "-f") les fichiers rsultants sur le serveur
#
use strict;
use Getopt::Std;
use Date::Calc qw(Today Delta_Days Add_Delta_Days);
use DBI;
use Net::FTP;

#
###############################################################################
#
my $urlftp = "serveur-perso-ftp";
my $loginftp = "utilisateur-ftp";
my $mdpftp = "mot-de-passe-ftp";
my $loginpg = "utilisateur-sql";
my $mdppg = "mot-de-passe-sql";
# date du classement
my $date;
my $anc;
my $moisc;
my $jourc;
my $delta;
# Rcuprer les options de la ligne de commande
our($opt_d, $opt_f, $opt_j, $opt_s);
if (getopts('fjsd:')) {
    print $opt_d . "\n" if ($opt_d);
} else {
    die "topbull07: usage: topbull07 [-d aaaa-mm-jj] [-f] [-s] [-j]\n";
}
# Ouvrir la base de donnes
#
my $dbh = DBI->connect('DBI:Pg:dbname=thebull',
		       $loginpg,
		       $mdppg,
		       {
			   AutoCommit => 0
		       }
		       ) or die "Erreur de connexion  la base: " . DBI->errstr;
#
# date de rfrence = dernire date enregistre dans les rsultats
#
if ($opt_d) {
    $date = $opt_d;
} else {
    my $select_date = $dbh->prepare('SELECT MAX(jour) from resultats07');
    die "Impossible de prparer la requte de date\n"
	unless defined $select_date;
    $select_date->execute();
    if ($select_date->rows == 0) {
	die "Pas de rsultat final\n";
    }
    $date = $select_date->fetchrow_array();
    $select_date->finish;
}
$anc = substr($date,0,4);
$moisc = substr($date,5,2);
$jourc = substr($date,8,2);
#
# date de comparaison
#
my ($sec,$min,$heure,$jour,$mois,$an,$js,$ja,$hiv);
my $fichier_top;
if ($opt_s) {
    $delta = -7;
} else {
    if ($opt_j) {
	$delta = -1;
    } else {
	# on dtermine la priode d'aprs le jour de la semaine
	($sec,$min,$heure,$jour,$mois,$an,$js,$ja,$hiv) = localtime(time);
	if ($js == 6 or $js == 0) {
	    # le samedi ou le dimanche on compare  la semaine d'avant
	    $delta = -7;
	} else {
	    if (($js == 1) or (($js == 2) and ($heure < 18))) {
		# le lundi, on compare au vendredi prcdent
		$delta = -3;
	    } else {
		# sinon on compare au jour prcdent
		$delta = -1;
	    }
	}
    }
}
if ($delta == -7) {
    $opt_s = "s";
} else {
    $opt_s = "";
}
($an,$mois,$jour) = Add_Delta_Days($anc,$moisc,$jourc, $delta );
$mois = $mois < 10 ? "0" . $mois : $mois;
$jour = $jour < 10 ? "0" . $jour : $jour;
my $datedavant = $an . "-" . $mois . "-" . $jour;
print "De " . $datedavant . "  " . $date . "\n";
#
# prparer la liste des portefeuilles
#
my $select_joueur = $dbh->prepare('SELECT portif,pseudo FROM joueurs07');
my @joueur; # rsultat de la requte $select_joueur
my $select_place = $dbh->prepare('SELECT place FROM resultats07 WHERE portif = ? AND jour = ?');
my $select_perf = $dbh->prepare('SELECT place,resultat FROM resultats07 WHERE portif = ? AND jour = ?');
my @perf; # rsultat de la requte $select_perf
die "Impossible de prparer les requtes\n"
    unless defined $select_joueur && defined $select_place
    && defined $select_perf;
#
my $top = -1;
my ($topjoueur, $topportif);
my $flop = 1;
my ($flopjoueur, $flopportif);
my $placeavant;
my $placeapres;
#
# Pour chaque joueur (et donc chaque portefeuille)
#
$select_joueur->execute();
while (@joueur = $select_joueur->fetchrow_array()) {
    # est-ce qu'on a la place d'arrive?
    $select_place->execute($joueur[0],$date);
    if ($select_place->rows == 0) {
	next;
    }
    $placeapres = $select_place->fetchrow_array();
    # est-ce qu'on a la place au dpart?
    $select_place->execute($joueur[0],$datedavant);
    if ($select_place->rows == 0) {
	next;
    }
    $placeavant = $select_place->fetchrow_array();
    $delta = $placeavant - $placeapres;
    if ($delta > $top) {
	$topportif = $joueur[0];
	$topjoueur = $joueur[1];
	$top = $delta;
    }
    if ($delta < $flop) {
	$flopportif = $joueur[0];
	$flopjoueur = $joueur[1];
	$flop = $delta;
    }
}
$select_place->finish;
$select_joueur->finish;
$fichier_top = "top" . $opt_s . substr($anc,2,2) . $moisc . $jourc . ".html";
open(TOP, ">" . $fichier_top);
print TOP "<table width=100%>\n<tr><h3>Le top et le flop ";
if ($opt_s) {
    print TOP "de la semaine</h3></tr>";
} else {
    print TOP "du jour</h3></tr>";
}
if ($top > -1) {
    $select_perf->execute($topportif,$datedavant);
    @perf = $select_perf->fetchrow_array();
    print TOP "<tr class=t2><td align=left><b>\U$topjoueur\E";
    print TOP "</b></td></tr>\n<tr class=t7><td class=t6>de la <b>" . $perf[0] . "</b>e place (";
    print TOP sprintf("%+.2f", $perf[1]) . ")";
    $select_perf->execute($topportif,$date);
    @perf = $select_perf->fetchrow_array();
    print TOP "  la <b>". $perf[0] . "</b>e place (";
    print TOP sprintf("%+.2f", $perf[1]) . ")</td></tr>\n";
} else {
    print "Top = " . $top . "\n";
}
if ($flop < 1) {
    $select_perf->execute($flopportif,$datedavant);
    @perf = $select_perf->fetchrow_array();
    print TOP "<tr class=t3><td align=left><b>\U$flopjoueur\E";
    print TOP "</b></td></tr>\n<tr class=t7><td class=t6>de la <b>" . $perf[0] . "</b>e place (";
    print TOP sprintf("%+.2f", $perf[1]) . ")";
    $select_perf->execute($flopportif,$date);
    @perf = $select_perf->fetchrow_array();
    print TOP "  la <b>" . $perf[0] . "</b>e place (";
    print TOP sprintf("%+.2f", $perf[1]) . ")</td></tr>\n";
} else {
    print "Flop = " . $flop . "\n";
}
print TOP "</table>\n";
$select_perf->finish;
close(TOP);
#
# fin de la boucle, on ferme!
#
$dbh->disconnect;
#
# Transfert des fichiers html vers le serveur (option -f)
#
if ($opt_f) {
    my $ftp = new Net::FTP($urlftp, Debug => 0, Passive => 1);
    $ftp->login($loginftp,$mdpftp);
    $ftp->cwd("/thebull");
    $ftp->put($fichier_top);
    $ftp->quit;
}
# c'est fini!
exit(0);
